package com.hcloud.microservice.org.controller;

import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.hcloud.microserver.commoncore.annotation.AuthOrgValidate;
import com.hcloud.microserver.commoncore.annotation.AuthValidate;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microservice.org.facade.client.OrgManagerService;
import com.hcloud.microservice.org.facade.entity.BaseOrg;
import com.hcloud.microservice.org.facade.forms.BaseAuthForm;
import com.hcloud.microservice.org.facade.forms.BaseOrgAuthForm;
import com.hcloud.microservice.org.facade.forms.BaseOrgForm;
import com.hcloud.microservice.org.facade.forms.BaseRoleAuthForm;
import com.hcloud.microservice.org.service.BaseOrgAuthorizationService;
import com.hcloud.microservice.org.service.BaseOrgService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author xiezt
 */
@RestController
@RequestMapping("/org")
public class OrgManagerController extends BaseController implements OrgManagerService {


    @Autowired
    private BaseOrgService baseOrgService;

    @Autowired
    private BaseOrgAuthorizationService baseOrgAuthorizationService;

    /**
     * 查询机构信息（分页查询）
     *
     * @param baseOrgForm
     * @return
     */
    @AuthOrgValidate
    @PostMapping("/searchByPage")
    @Override
    public ResultVO searchOrgInfoByPage(@RequestBody BaseOrgForm baseOrgForm) {
       PageInfo pageInfo =  baseOrgService.searchOrgInfoByPage(baseOrgForm);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    /**
     * 添加机构信息
     *
     * @param baseOrgForm
     * @return
     */
    @AuthOrgValidate
    @PostMapping("/addOrgInfo")
    @Override
    public ResultVO addOrgInfo(@RequestBody BaseOrgForm baseOrgForm) {
        BaseOrgForm queryForm = new BaseOrgForm();
        queryForm.setOrgCode(baseOrgForm.getOrgCode());
       PageInfo pageInfo =  baseOrgService.searchOrgInfoByPage(queryForm);
       if(null!=pageInfo.getList() && pageInfo.getTotal()==0){
           BaseOrg baseOrg = new BaseOrg();
           BeanUtils.copyProperties(baseOrgForm,baseOrg);
           Integer i = baseOrgService.saveSelective(baseOrg);
           if(i==0){return new ResultVO(ResultEnum.SYS_ERROR);}
           return success();
       }
        return new ResultVO(ResultEnum.ORG_IS_NOT);
    }

    /**
     * 更新机构信息
     *
     * @param baseOrgForm
     * @return
     */
    @AuthOrgValidate
    @PostMapping("/updateOrgInfo")
    @Override
    public ResultVO updateOrgInfo(@RequestBody BaseOrgForm baseOrgForm) {
        BaseOrg baseOrg = new BaseOrg();
        BeanUtils.copyProperties(baseOrgForm,baseOrg);
        Integer i =  baseOrgService.modifyByPrimaryKeySelective(baseOrg);
        if(i==0){return new ResultVO(ResultEnum.SYS_ERROR);}
        return success();
    }

    /**
     * 查询机构明细信息
     *
     * @param guid
     * @return
     */
    @AuthOrgValidate
    @PostMapping("/searchById")
    @Override
    public ResultVO searchOrgInfoById(@RequestParam("guid") String guid) {
        BaseOrg bean = baseOrgService.queryByPrimaryKey(guid);
        return success(bean);
    }

    /**
     * 分配机构权限
     *
     * @param orgGuid
     * @param list
     * @return
     */
    @AuthOrgValidate
    @PostMapping("/distrOrgAuth")
    @Override
    public ResultVO distributionOrgAuthInfo(@RequestParam("orgGuid") String orgGuid,@RequestBody List<BaseOrgAuthForm> list) {
        int i = baseOrgAuthorizationService.batchOrgAuthInfo(list,orgGuid);
        if(i==0){return new ResultVO(ResultEnum.SYS_ERROR);}
        return success();
    }

    /**
     * 查询已分配的机构权限
     *
     * @param orgGuid
     * @return
     */
    @AuthOrgValidate
    @GetMapping("/searchOrgAuthInfo")
    @Override
    public ResultVO searchOrgAuthDistributioned(@RequestParam("orgGuid") String orgGuid) {
        List<BaseOrgAuthForm> list = baseOrgService.searchAllOrgAuthInfo(orgGuid);
        BaseOrgAuthForm baseOrgAuthForm = convertAuthMenu(list);
         return success(baseOrgAuthForm);
    }


    private BaseOrgAuthForm convertAuthMenu( List<BaseOrgAuthForm> list){
        BaseOrgAuthForm baseRoleAuthForm = new BaseOrgAuthForm();
        if(null!= list && list.size()>0){
            for (BaseOrgAuthForm bean: list) {
                if("#".equals(bean.getParentGuid())){
                    BeanCopyUtil.copyBean(bean,baseRoleAuthForm);
                    baseRoleAuthForm.setSubMenuList(getSubMenuList(bean,list));
                    break;
                }
            }
        }
        return baseRoleAuthForm;
    }

        private List<BaseOrgAuthForm> getSubMenuList(BaseOrgAuthForm baseOrgAuthForm, List<BaseOrgAuthForm> list){
        List<BaseOrgAuthForm> subList = Lists.newArrayList();
        List<BaseOrgAuthForm> resultList = Lists.newArrayList();
        //TODO 查询子菜单
        for(BaseOrgAuthForm en:list){
            String parentId = baseOrgAuthForm.getId();
            if(parentId.equals(en.getParentGuid())){
                subList.add(en);
            }
        }
        for(BaseOrgAuthForm bean:subList){
            if(bean.getNodeType()==0){
                bean.setSubMenuList(getSubMenuList(bean,list));
            }
            resultList.add(bean);
        }
        return resultList;
    }
}
