package com.hcloud.microservice.org.controller;

import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.hcloud.microserver.commoncore.annotation.AuthOrgValidate;
import com.hcloud.microserver.commoncore.annotation.AuthValidate;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microservice.org.facade.client.RoleManagerService;
import com.hcloud.microservice.org.facade.entity.BaseRole;
import com.hcloud.microservice.org.facade.forms.BaseRoleAuthForm;
import com.hcloud.microservice.org.facade.forms.BaseRoleForm;
import com.hcloud.microservice.org.service.BaseRoleService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author xiezt
 */
@Slf4j
@RestController
@RequestMapping("/role")
public class RoleManagerController extends BaseController implements RoleManagerService {

    @Autowired
    private BaseRoleService baseRoleService;

    /**
     * 分页查询角色信息
     *
     * @param baseRoleForm
     * @return
     */
    @AuthOrgValidate
    @PostMapping("/searchByPage")
    @Override
    public ResultVO searchRoleInfoByPage(@RequestBody BaseRoleForm baseRoleForm) {
        PageInfo pageInfo = baseRoleService.searchRoleInfoByPage(baseRoleForm);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    /**
     * 添加角色信息
     *
     * @param baseRoleForm
     * @return
     */
    @AuthOrgValidate
    @PostMapping("/addRoleInfo")
    @Override
    public ResultVO addRoleInfo(@RequestBody BaseRoleForm baseRoleForm) {
        BaseRole baseRole = new BaseRole();
        BeanUtils.copyProperties(baseRoleForm,baseRole);
        int i =baseRoleService.saveSelective(baseRole);
        if(i==0){return new ResultVO(ResultEnum.SYS_ERROR);}
        return success();
    }

    /**
     * 更新角色信息
     *
     * @param baseRoleForm
     * @return
     */
    @AuthOrgValidate
    @PostMapping("/updateRoleInfo")
    @Override
    public ResultVO updateRoleInfo(@RequestBody BaseRoleForm baseRoleForm) {
        BaseRole baseRole = new BaseRole();
        BeanUtils.copyProperties(baseRoleForm,baseRole);
        int i = baseRoleService.modifyByPrimaryKeySelective(baseRole);
        if(i==0){return new ResultVO(ResultEnum.SYS_ERROR);}
        return success();
    }

    /**
     * 查询角色明细信息
     *
     * @param guid
     * @return
     */
    @AuthOrgValidate
    @PostMapping("/searchById")
    @Override
    public ResultVO searchInfoById(@RequestParam("guid") String guid) {
        BaseRole baseRole = baseRoleService.queryByPrimaryKey(guid);
        return success(baseRole);
    }

    /**
     * 查询已分配角色权限信息
     *
     * @param roleId
     * @param orgGuid
     * @return
     */
    @AuthOrgValidate
    @PostMapping("/searchRoleAuthInfo")
    @Override
    public ResultVO searchRoleAuthInfo(@RequestParam("roleId") String roleId, @RequestParam("orgGuid") String orgGuid) {
        List<BaseRoleAuthForm> list = baseRoleService.searchRoleAuthInfo(roleId,orgGuid);
        return success(convertRoleMenu(list));
    }

    /**
     * 分配角色权限
     *
     * @param list
     * @return
     */
    @AuthOrgValidate
    @PostMapping("/distribedRoleAuth")
    @Override
    public ResultVO distributionRoleAuth(@RequestParam("roleId") String roleId,
                                         @RequestParam("orgGuid") String orgGuid,
                                         @RequestBody List<BaseRoleAuthForm> list) {
        int i = baseRoleService.batchSaveRoleAuthInfo(list,roleId,orgGuid);
        if(i==0){return new ResultVO(ResultEnum.SYS_ERROR);}
        return success();
    }




    private BaseRoleAuthForm convertRoleMenu(List<BaseRoleAuthForm> list){
        BaseRoleAuthForm baseRoleAuthForm = new BaseRoleAuthForm();
        if(null!= list && list.size()>0){
            for (BaseRoleAuthForm bean: list) {
                if("#".equals(bean.getParentGuid())){
                    BeanCopyUtil.copyBean(bean,baseRoleAuthForm);
                    baseRoleAuthForm.setSubMenuList(getSubMenuList(bean,list));
                    break;
                }
            }
        }
        return baseRoleAuthForm;
    }

        private List<BaseRoleAuthForm> getSubMenuList(BaseRoleAuthForm baseRoleAuthForm,List<BaseRoleAuthForm> list){
        List<BaseRoleAuthForm> subList = Lists.newArrayList();
        List<BaseRoleAuthForm> resultList = Lists.newArrayList();
        //TODO 查询子菜单
        for(BaseRoleAuthForm en:list){
            String parentId = baseRoleAuthForm.getId();
            if(parentId.equals(en.getParentGuid())){
                subList.add(en);
            }
        }
        for(BaseRoleAuthForm bean:subList){
            if(bean.getNodeType()==0){
                bean.setSubMenuList(getSubMenuList(bean,list));
            }
            resultList.add(bean);
        }
       return resultList;
    }

}
