package com.hcloud.microservice.org.controller;

import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.hcloud.microserver.commoncore.annotation.AuthOrgValidate;
import com.hcloud.microserver.commoncore.annotation.AuthValidate;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.base.Token;
import com.hcloud.microserver.commoncore.constant.GlobleConstant;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.service.RedisUtils;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.Md5Util;
import com.hcloud.microserver.commoncore.util.TokenUtil;
import com.hcloud.microservice.org.facade.client.UserManagerService;
import com.hcloud.microservice.org.facade.entity.BaseOrg;
import com.hcloud.microservice.org.facade.entity.BaseUser;
import com.hcloud.microservice.org.facade.forms.BaseUserAuthForm;
import com.hcloud.microservice.org.facade.forms.BaseUserForm;
import com.hcloud.microservice.org.service.BaseOrgService;
import com.hcloud.microservice.org.service.BaseUserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.net.util.Base64;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.io.UnsupportedEncodingException;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author xiezt
 */
@Slf4j
@RestController
@RequestMapping("/user")
public class UserManagerController extends BaseController implements UserManagerService {

    private static final String PERMISSION_KEY = "url";

    private static final String PERMISSION_ID = "perId";

    private static final String PERMISSION_NAME = "perName";

    private static final String PERMISSION_IS_CHECKD="isChecked";

    @Autowired
    private BaseUserService baseUserService;

    @Autowired
    private BaseOrgService baseOrgService;


    @Autowired
    private RedisUtils redisUtils;


    /**
     * 分页查询用户信息
     * @param baseUserForm
     * @return
     */
    @AuthOrgValidate
    @PostMapping("/searchByPage")
    @Override
    public ResultVO searchUserInfoByPage(@RequestBody BaseUserForm baseUserForm) {
        PageInfo pageInfo =baseUserService.searchUserInfoByPage(baseUserForm);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    /**
     * 添加用户
     * @param baseUserForm
     * @return
     */
    @AuthOrgValidate
    @PostMapping("/addUserInfo")
    @Override
    public ResultVO addUserInfo(@RequestBody BaseUserForm baseUserForm) {
        log.info("================>{}",baseUserForm);
        String account = baseUserForm.getAccount();
        boolean repeatFlag = baseUserService.searchRepeatUserInfo(account);
        if(!repeatFlag){
            int i = baseUserService.addUserInfoForAll(baseUserForm);
            if(i==-2){return new ResultVO(701,"角色未配置权限！");}
            if(i==0){return new ResultVO(ResultEnum.SYS_ERROR);}
            return success();
        }
        return new ResultVO(609,"账号重复");
    }

    /**
     * 更新用户信息
     * @param baseUserForm
     * @return
     */
    @AuthOrgValidate
    @PostMapping("/updateUserInfo")
    @Override
    public ResultVO updateUserInfo(@RequestBody BaseUserForm baseUserForm) {
        if (!this.baseUserService.isAccountsValid(baseUserForm)) {
            return new ResultVO(104, "修改失败, 用户账户已经存在");
        }
        if(baseUserForm.getRoleId()==null){
            BaseUser user  = new BaseUser();
            BeanCopyUtil.copyBean(baseUserForm,user);
            String oldPasswd = baseUserForm.getOldPasswd();
            if(oldPasswd!=null && !"".equals(oldPasswd)){
                String minOldPasswd = this.checkoutPwd(oldPasswd);
                BaseUser tmpUser = baseUserService.queryByPrimaryKey(user.getGuid());
                if(!Md5Util.md5(minOldPasswd).equalsIgnoreCase(tmpUser.getPasswd())){
                    return new ResultVO(ResultEnum.SYS_PASSWD_ERROR);
                }
            }
            int i = baseUserService.modifyByPrimaryKeySelective(user);
            if(i==0){return new ResultVO(ResultEnum.SYS_ERROR);}
        }else{
            int i = baseUserService.modifyUserInfoForAll(baseUserForm);
            if(i==0){return new ResultVO(ResultEnum.SYS_ERROR);}
        }
        return success();
    }

    /**
     * 查询用户权限
     * @param userId
     * @param orgGuid
     * @return
     */
    @AuthOrgValidate
    @GetMapping("/searchUserAuthInfo")
    @Override
    public ResultVO searchUserAuthInfo(@RequestParam("userId") String userId, @RequestParam("orgGuid") String orgGuid) {
       List<BaseUserAuthForm> list = baseUserService.searchUserAuthInfo(userId,orgGuid);
        return success(list);
    }

    /**
     * 分配权限
     *
     * @param userId
     * @param orgGuid
     * @param list
     * @return
     */
    @AuthOrgValidate
    @PostMapping("/distribedUserAuthInfo")
    @Override
    public ResultVO distributionUserAuthInfo(@RequestParam("userId") String userId,
                                             @RequestParam("orgGuid") String orgGuid,
                                             @RequestBody List<BaseUserAuthForm> list) {
        BaseUserForm baseUserForm = new BaseUserForm();
        baseUserForm.setGuid(userId);
        baseUserForm.setFkOrgGuid(orgGuid);
        baseUserForm.setAuthList(list);
        baseUserService.distributionUserAuthInfo(baseUserForm);
        return success();
    }

    /**
     * 用户登录系统
     *
     * @param account
     * @param password
     * @return
     */
    @AuthOrgValidate
    @PostMapping("/loginSys")
    @Override
    public ResultVO loginSystem(@RequestParam("account") String account,@RequestParam("password") String password) {
        Map reslutMap = Collections.emptyMap();
        Token token = null;
        BaseUserForm baseUserForm =  baseUserService.searchUserInfoByAccount(account,password);
        if(null!=baseUserForm){
            ResultEnum resultEnum = baseUserForm.getResultEnum();
            if(resultEnum.getCode()!=0){return new ResultVO(resultEnum);}
            reslutMap =  new HashMap();
            String userId = baseUserForm.getGuid();
            String orgGuid = baseUserForm.getFkOrgGuid();
            BaseOrg baseOrg = baseOrgService.queryByPrimaryKey(orgGuid);
            List<BaseUserAuthForm> authList = baseUserService.searchUserAuthInfo(userId,baseUserForm.getFkOrgGuid());
            token = TokenUtil.createJwtToken(userId, GlobleConstant.CURRENT_USER);
            reslutMap.put(GlobleConstant.ACCESS_TOKEN,token.getAccessToken());
            reslutMap.put(GlobleConstant.AUTH_LIST, authList);
            reslutMap.put(GlobleConstant.AUTH_MAP,convertAuthListToMap(authList));
            reslutMap.put(GlobleConstant.AUTH_MENN_OBJ, convertMenuList(authList));
            reslutMap.put(GlobleConstant.ORG_GUID,baseUserForm.getFkOrgGuid());
            reslutMap.put(GlobleConstant.CURRENT_USER, baseUserForm);
            reslutMap.put(GlobleConstant.USER_GUID,baseUserForm.getGuid());
            reslutMap.put(GlobleConstant.ORG_OBJECT,baseOrg);
            redisUtils.setValue(token.getAccessToken(), reslutMap, 2, TimeUnit.HOURS);
            return success(reslutMap);
        }
        return new ResultVO(ResultEnum.FAIL_LOGIN_LOGIN_NAME);
    }

    /**
     * 获取用户菜单信息
     *
     * @param userId
     * @param orgGuid
     * @return
     */
    @AuthOrgValidate
    @PostMapping("/getUserMenus")
    @Override
    public ResultVO getUserMenusInfo(@RequestParam("userId") String userId,
                                     @RequestParam("orgGuid") String orgGuid) {
        List<BaseUserAuthForm> list = baseUserService.getUserMenusInfo(userId,orgGuid);
            return success(list);
    }

    /**
     * 获取菜单子功能信息
     *
     * @param baseUserAuthForm
     * @return
     */
    @AuthOrgValidate
    @PostMapping("/getUserSubAuth")
    @Override
    public ResultVO getSubUserAuthInfo(@RequestBody BaseUserAuthForm baseUserAuthForm) {
       List<BaseUserAuthForm> list =  baseUserService.getSubAuthById(baseUserAuthForm.getFkUserGuid(),
                baseUserAuthForm.getFkOrgGuid(),baseUserAuthForm.getFkAuthGuid());
        return success(list);
    }

    /**
     * 转换用户权限变为菜单显示
     * @param list
     * @return
     */
    private BaseUserAuthForm convertMenuList(List<BaseUserAuthForm> list){
        BaseUserAuthForm baseUserAuthForm = new BaseUserAuthForm();
        if(null!= list && list.size()>0){
            for (BaseUserAuthForm bean:list) {
                if("#".equalsIgnoreCase(bean.getParentGuid())){
                    BeanCopyUtil.copyBean(bean,baseUserAuthForm);
                    baseUserAuthForm.setSubMenuList(getSubMenuListById(bean));
                    break;
                  }
            }
        }
        log.info("baseUserAuthForm=====================>{}",baseUserAuthForm);
        return baseUserAuthForm;
    }

    private List<BaseUserAuthForm> getSubMenuListById(BaseUserAuthForm bean){
        List<BaseUserAuthForm> resultList = Lists.newArrayList();
        List<BaseUserAuthForm> tmpList = baseUserService.getSubAuthById(bean.getFkUserGuid(),
                bean.getFkOrgGuid(), bean.getId());
        for (BaseUserAuthForm entity: tmpList) {
            if(entity.getIsChecked()==1 && entity.getNodeType()==0){
                entity.setSubMenuList(getSubMenuListById(entity));
            }
            resultList.add(entity);
        }
        return resultList;
    }

    private List<Map<String,Object>> convertAuthListToMap(List<BaseUserAuthForm> list){
        List<Map<String,Object>> resultList = Lists.newArrayList();
        if(null!=list && list.size()>0){
            for (BaseUserAuthForm bean:list) {
                Map<String,Object> map = new HashMap<>();
                map.put(PERMISSION_KEY,bean.getAuthStr());
                map.put(PERMISSION_ID,bean.getId());
                map.put(PERMISSION_NAME,bean.getFunName());
                map.put(PERMISSION_IS_CHECKD,bean.getIsChecked());
                resultList.add(map);
            }
        }
        return resultList;
    }

    /**
     * 登录解密方式
     * @param pwd
     * @return
     */
    private String checkoutPwd(String pwd)  {
        //需要进行对应的接密操作
        //1.获取盐值
        String salt = pwd.substring(0, 5);
        String key = "b8d11ee289394be688ef3a4f6968efed";
        String time1 = Md5Util.md5((key).substring(0, 8)).substring(0, 5);
        //做超时判断
        if (!time1.equalsIgnoreCase(salt)) {
            throw new RuntimeException();
        }
        //2.去除盐值
        String replace = pwd.replace(salt, "");
        //3.还原base64前的密码值
        try {
            return new String(Base64.decodeBase64(replace), "utf-8");
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException();
        }
    }
}
