package com.hcloud.microservice.org.service;

import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.base.BaseServie;
import com.hcloud.microservice.org.facade.entity.BaseUser;
import com.hcloud.microservice.org.facade.forms.BaseOrgAuthForm;
import com.hcloud.microservice.org.facade.forms.BaseRoleUserForm;
import com.hcloud.microservice.org.facade.forms.BaseUserAuthForm;
import com.hcloud.microservice.org.facade.forms.BaseUserForm;

import java.util.List;

/**
 * @author xiezt
 */
public interface BaseUserService extends BaseServie<BaseUser,String>  {

    /**
     * 批量插入用户权限信息
     * @param list
     * @param userId
     * @param orgGuid
     * @return
     */
    int batchSaveUserAuthInfo(String userId,String orgGuid,List<BaseUserAuthForm> list);

    /**
     * 分页查询用户信息
     * @param baseUserForm
     * @return
     */
    PageInfo searchUserInfoByPage(BaseUserForm baseUserForm);

    /**
     * 通过用户id查询用户权限信息
     * @param userId
     * @param orgGuid
     * @return
     */
    List<BaseUserAuthForm> searchUserAuthInfo(String userId, String orgGuid);

    /**
     * 通过用户id删除用户权限信息
     * @param userId
     * @param orgGuid
     * @return
     */
    int delUserAuthInfoByUserId(String userId,String orgGuid);

    /**
     * 查询用户角色信息，通过用户查询角色，通过角色查用户
     * @param baseRoleUserForm
     * @return
     */
    PageInfo searchRoleUserInfoById(BaseRoleUserForm baseRoleUserForm);


    /**
     * 分配用户角色信息
     * @param baseUserForm
     */
    void distributionUserAuthInfo(BaseUserForm baseUserForm);

    /**
     * 查询用户信息
     * @param account
     * @param password
     * @return
     */
    BaseUserForm searchUserInfoByAccount(String account,String password);

    /**
     * 查询用户菜单信息
     * @param userId
     * @param orgGuid
     * @return
     */
    List<BaseUserAuthForm> getUserMenusInfo(String userId,String orgGuid);


    /**
     * 查询功能权限通过菜单权限
     * @param userId
     * @param orgGuid
     * @param authId
     * @return
     */
    List<BaseUserAuthForm> getSubAuthById(String userId,String orgGuid,String authId);

    /**
     * 查询重复用户信息
     * @param account
     * @return
     */
    boolean searchRepeatUserInfo(String account);


    /**
     * 新增用户信息（包含用户属于哪个角色，用户拥有的权限）
     * @param baseUserForm
     * @return
     */
    int addUserInfoForAll(BaseUserForm baseUserForm);

    /**
     * 修改用户信息（包含用户属于哪个角色，用户拥有的权限）
     * @param baseUserForm
     * @return
     */
    int modifyUserInfoForAll(BaseUserForm baseUserForm);

    /**
     * 查找用户账户是否存在
     * 排除自身账户 如果没有的accout返回true
     * @param baseUserForm
     * @return false
     */
    boolean isAccountsValid(BaseUserForm baseUserForm);
}
