package com.hcloud.microservice.org.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microservice.org.bo.BaseAuthBO;
import com.hcloud.microservice.org.dao.BaseAuthorizationMapper;
import com.hcloud.microservice.org.facade.entity.BaseAuthorization;
import com.hcloud.microservice.org.facade.forms.BaseAuthForm;
import com.hcloud.microservice.org.service.BaseAuthorizationService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

@Service("baseAuthorizationService")
public class BaseAuthorizationServiceImpl implements BaseAuthorizationService {

    @Resource
    private BaseAuthorizationMapper baseAuthorizationMapper;

    @Override
    public int delByPrimaryKey(String guid) {
        return baseAuthorizationMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(BaseAuthorization record) {
        record.setGuid(GetUUID.getUUID());
        return baseAuthorizationMapper.insert(record);
    }

    @Override
    public int saveSelective(BaseAuthorization record) {
        record.setGuid(GetUUID.getUUID());
        return baseAuthorizationMapper.insertSelective(record);
    }

    @Override
    public BaseAuthorization queryByPrimaryKey(String guid) {
        return baseAuthorizationMapper.selectByPrimaryKey(guid);
    }

    @Override
    public int modifyByPrimaryKeySelective(BaseAuthorization record) {
        return baseAuthorizationMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(BaseAuthorization record) {
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(BaseAuthorization record) {
        return baseAuthorizationMapper.updateByPrimaryKey(record);
    }

    /**
     * 分页查询权限信息
     *
     * @param baseAuthForm 页面交互信息
     * @return 权限分页信息
     */
    @Override
    public PageInfo searchBaseAuthInfoByPage(BaseAuthForm baseAuthForm) {
        PageBean pageBean = new PageBean(baseAuthForm);
        PageHelper.offsetPage(baseAuthForm.getPageNo(),baseAuthForm.getPageSize());
        Page<BaseAuthBO> page =  baseAuthorizationMapper.searchBaseAuthInfoByPage(pageBean);
        List<BaseAuthForm> list = BeanCopyUtil.convertList(page,BaseAuthForm.class);
        return new PageInfo(list);
    }

    /**
     * 查询菜单权限信息（存在url地址的权限）
     *
     * @return
     */
    @Override
    public List<BaseAuthBO> searchMenuAuthInfo() {
        return baseAuthorizationMapper.searchMenuAuthInfo();
    }
}
