package com.hcloud.microservice.org.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microservice.org.bo.BaseAuthBO;
import com.hcloud.microservice.org.dao.BaseAuthorizationMapper;
import com.hcloud.microservice.org.dao.BaseOrgAuthorizationMapper;
import com.hcloud.microservice.org.dao.BaseOrgMapper;
import com.hcloud.microservice.org.facade.entity.BaseOrg;
import com.hcloud.microservice.org.facade.forms.BaseOrgAuthForm;
import com.hcloud.microservice.org.facade.forms.BaseOrgForm;
import com.hcloud.microservice.org.service.BaseOrgService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
@Slf4j
@Service("baseOrgService")
public class BaseOrgServiceImpl implements BaseOrgService {

    @Resource
    private BaseOrgMapper baseOrgMapper;

    @Resource
    private BaseOrgAuthorizationMapper baseOrgAuthorizationMapper;

    @Resource
    private BaseAuthorizationMapper baseAuthorizationMapper;

    @Override
    public int delByPrimaryKey(String guid) {
        return baseOrgMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(BaseOrg record) {
        record.setGuid(GetUUID.getUUID());
        return baseOrgMapper.insert(record);
    }

    @Override
    public int saveSelective(BaseOrg record) {
        record.setGuid(GetUUID.getUUID());
        return baseOrgMapper.insertSelective(record);
    }

    @Override
    public BaseOrg queryByPrimaryKey(String guid) {
        return baseOrgMapper.selectByPrimaryKey(guid);
    }

    @Override
    public int modifyByPrimaryKeySelective(BaseOrg record) {
        return baseOrgMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(BaseOrg record) {
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(BaseOrg record) {
        return baseOrgMapper.updateByPrimaryKey(record);
    }

    /**
     * 分页查询机构信息
     *
     * @param baseOrgForm
     * @return
     */
    @Override
    public PageInfo searchOrgInfoByPage(BaseOrgForm baseOrgForm) {
        PageBean pageBean = new PageBean(baseOrgForm);
        PageHelper.startPage(baseOrgForm.getPageNo(),baseOrgForm.getPageSize());
        Page<BaseOrg> page =  baseOrgMapper.searchOrgInfoByPage(pageBean);
        return new PageInfo(page);
    }

    /**
     * 机构管理查询中查询机构权限
     *
     * @param orgGuid
     * @return
     */
    @Override
    public List<BaseOrgAuthForm> searchAllOrgAuthInfo(String orgGuid) {
        List<BaseOrgAuthForm> resultList = new ArrayList<>();
        //TODO 基础权限信息
        Page<BaseAuthBO> page = baseAuthorizationMapper.searchAllBaseAuthInfo();
        //TODO 机构权限标记
        List<String> orgList = baseOrgAuthorizationMapper.searchOrgAuthStringByOrgId(orgGuid);
        //TODO 处理权限信息
        if(null!= page && page.size()>0){
            for (BaseAuthBO bo:page) {
                String authGuid = bo.getGuid();
                BaseOrgAuthForm baseOrgAuthForm = new BaseOrgAuthForm();
                baseOrgAuthForm.setId(authGuid);
                baseOrgAuthForm.setFkProductGuid(bo.getFkProductGuid());
                baseOrgAuthForm.setFkOrgGuid(orgGuid);
                baseOrgAuthForm.setParentGuid(bo.getParentGuid());
                baseOrgAuthForm.setFkAuthGuid(bo.getGuid());
                baseOrgAuthForm.setFunName(bo.getFunName());
                baseOrgAuthForm.setAuthStr(bo.getAuthStr());
                baseOrgAuthForm.setFileUrl(bo.getFileUrl());
                baseOrgAuthForm.setIco(bo.getIco());
                baseOrgAuthForm.setOrderby(bo.getOrderby());
                baseOrgAuthForm.setNodeType(bo.getNodeType());
                baseOrgAuthForm.setIsForbid(bo.getIsForbid());
                if(orgList.contains(authGuid)){
                    log.info("funName===========================>{}",bo.getFunName());
                    baseOrgAuthForm.setIsChecked(1);
                }
                resultList.add(baseOrgAuthForm);
            }
        }
        return resultList;
    }

    /**
     * 获取机构菜单权限
     *
     * @param OrgGuid
     * @return
     */
    @Override
    public List<BaseOrgAuthForm> searchMenuAuthInfo(String OrgGuid) {
        return null;
    }
}
