package com.hcloud.microservice.org.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.commoncore.constant.GlobleConstant;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microservice.org.bo.BaseOrgAuthBO;
import com.hcloud.microservice.org.bo.BaseRoleAuthBO;
import com.hcloud.microservice.org.bo.BaseRoleUserBO;
import com.hcloud.microservice.org.dao.BaseOrgAuthorizationMapper;
import com.hcloud.microservice.org.dao.BaseRoleAuthorizationMapper;
import com.hcloud.microservice.org.dao.BaseRoleMapper;
import com.hcloud.microservice.org.dao.BaseUserMapper;
import com.hcloud.microservice.org.facade.entity.BaseOrgAuthorization;
import com.hcloud.microservice.org.facade.entity.BaseRole;
import com.hcloud.microservice.org.facade.forms.BaseRoleAuthForm;
import com.hcloud.microservice.org.facade.forms.BaseRoleForm;
import com.hcloud.microservice.org.facade.forms.BaseUserAuthForm;
import com.hcloud.microservice.org.service.BaseRoleService;
import com.hcloud.microservice.org.service.BaseUserService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author xiezt
 */
@Service("baseRoleService")
public class BaseRoleServiceImpl implements BaseRoleService {

    @Resource
    private BaseRoleMapper baseRoleMapper;

    @Resource
    private BaseRoleAuthorizationMapper baseRoleAuthorizationMapper;

    @Resource
    private BaseOrgAuthorizationMapper baseOrgAuthorizationMapper;

    @Autowired
    private BaseUserService baseUserService;

    @Override
    public int delByPrimaryKey(String guid) {
        return baseRoleMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(BaseRole record) {
        record.setGuid(GetUUID.getUUID());
        return baseRoleMapper.insert(record);
    }

    @Override
    public int saveSelective(BaseRole record) {
        record.setGuid(GetUUID.getUUID());
        return baseRoleMapper.insertSelective(record);
    }

    @Override
    public BaseRole queryByPrimaryKey(String guid) {
        return baseRoleMapper.selectByPrimaryKey(guid);
    }

    @Override
    public int modifyByPrimaryKeySelective(BaseRole record) {
        return baseRoleMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(BaseRole record) {
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(BaseRole record) {
        return baseRoleMapper.updateByPrimaryKey(record);
    }

    /**
     * 分页查询角色信息
     *
     * @param baseRoleForm
     * @return
     */
    @Override
    public PageInfo searchRoleInfoByPage(BaseRoleForm baseRoleForm) {
        PageBean pageBean = new PageBean(baseRoleForm);
        PageHelper.startPage(baseRoleForm.getPageNo(),baseRoleForm.getPageSize());
        Page<BaseRole> page =  baseRoleMapper.searchRoleInfoByPage(pageBean);
        return new PageInfo(page);
    }

    /**
     * 批量插入角色权限信息
     *
     * @param formsList
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int batchSaveRoleAuthInfo(List<BaseRoleAuthForm> formsList,String roleId,String orgGuid) {
        List<BaseRoleAuthForm> insertList = Lists.newArrayList();
        if(formsList.size()>0){
            baseRoleAuthorizationMapper.delRoleAuthByRoleId(roleId,orgGuid);
            for(BaseRoleAuthForm bean:formsList){
                BaseRoleAuthForm authForm = new BaseRoleAuthForm();
                authForm.setFkOrgGuid(bean.getFkOrgGuid());
                authForm.setFkProductGuid(bean.getFkProductGuid());
                authForm.setFkRoleGuid(roleId);
                authForm.setFkAuthGuid(bean.getId());
                insertList.add(authForm);
            }
        }
        int i = baseRoleAuthorizationMapper.batchSaveRoleAuthInfo(insertList);
        if(i>0){
            //TODO 查询角色对应用户，更新用户权限
            List<BaseRoleUserBO> userList = baseRoleMapper.searchUserInfoByRoleId(roleId);
            if(null!=userList && userList.size()>0){
                for (BaseRoleUserBO bean:userList) {
                    List<BaseUserAuthForm> userAuthFormList = getUserAuthInfo(bean.getUserId(),insertList);
                    baseUserService.batchSaveUserAuthInfo(bean.getUserId(),bean.getFkOrgGuid(),userAuthFormList);
                }
            }

        }
        return i;
    }

    private List<BaseUserAuthForm> getUserAuthInfo(String userId,List<BaseRoleAuthForm> insertList){
        if(null!=insertList && insertList.size()>0){
            List<BaseUserAuthForm> list = Lists.newArrayList();
            for (BaseRoleAuthForm bean:insertList) {
                BaseUserAuthForm tmp = new BaseUserAuthForm();
                tmp.setFkOrgGuid(bean.getFkOrgGuid());
                tmp.setFkUserGuid(userId);
                tmp.setFkProductGuid(GlobleConstant.PRODUCT_GUID);
                tmp.setId(bean.getFkAuthGuid());
                list.add(tmp);
            }
            return list;
        }
        return null;
    }


    /**
     * 通过角色Id删除角色权限信息
     *
     * @param roleId
     * @param orgGuid
     * @return
     */
    @Override
    public int delRoleAuthInfoByRoleId(String roleId, String orgGuid) {
        return baseRoleAuthorizationMapper.delRoleAuthByRoleId(roleId,orgGuid);
    }

    /**
     * 查询机构权限信息
     *
     * @param roleId
     * @param orgGuid
     * @return
     */
    @Override
    public List<BaseRoleAuthForm> searchRoleAuthInfo(String roleId, String orgGuid) {
        List<BaseRoleAuthForm> resultList = new ArrayList<>();
        //TODO 查询机构权限信息
        List<BaseOrgAuthBO> orgAuthList = baseOrgAuthorizationMapper.searchAllOrgAuthInfo(orgGuid);
        //TODO 查询角色权限信息
        List<String> list = baseRoleAuthorizationMapper.searchRoleAuthByRoleId(roleId,orgGuid);
        if(null!=orgAuthList && orgAuthList.size()>0){
            for (BaseOrgAuthBO bo:orgAuthList) {
                String authStr = bo.getId();
                BaseRoleAuthForm baseRoleAuthForm = new BaseRoleAuthForm();
                BeanUtils.copyProperties(bo,baseRoleAuthForm);
                if(list.contains(authStr)){
                    baseRoleAuthForm.setIsChecked(1);
                }else{
                    baseRoleAuthForm.setIsChecked(0);
                }
                resultList.add(baseRoleAuthForm);
            }
        }
        return resultList;
    }

    /**
     * 查询角色机构信息(不与机构权限信息比对)
     *
     * @param roleId
     * @param orgGuid
     * @return
     */
    @Override
    public List<BaseRoleAuthForm> searchOnlyRoleAuthInfo(String roleId, String orgGuid) {
        List<BaseRoleAuthForm> resultList = new ArrayList<>();
        //TODO 查询机构权限信息
        List<BaseOrgAuthBO> orgAuthList = baseOrgAuthorizationMapper.searchAllOrgAuthInfo(orgGuid);
        //TODO 查询角色权限信息
        List<String> list = baseRoleAuthorizationMapper.searchRoleAuthByRoleId(roleId,orgGuid);
        if(null!=orgAuthList && orgAuthList.size()>0){
            for (BaseOrgAuthBO bo:orgAuthList) {
                String authStr = bo.getId();
                if(list.contains(authStr)){
                    BaseRoleAuthForm baseRoleAuthForm = new BaseRoleAuthForm();
                    BeanUtils.copyProperties(bo,baseRoleAuthForm);
                    resultList.add(baseRoleAuthForm);
                }
            }
        }
        return resultList;
    }

}
