package com.hcloud.microservice.org.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.commoncore.constant.GlobleConstant;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microserver.commoncore.util.Md5Util;
import com.hcloud.microserver.commoncore.util.string.StringUtils;
import com.hcloud.microservice.org.bo.BaseOrgAuthBO;
import com.hcloud.microservice.org.bo.BaseRoleUserBO;
import com.hcloud.microservice.org.bo.BaseUserBO;
import com.hcloud.microservice.org.dao.BaseOrgAuthorizationMapper;
import com.hcloud.microservice.org.dao.BaseRoleUserMapper;
import com.hcloud.microservice.org.dao.BaseUserAuthorizationMapper;
import com.hcloud.microservice.org.dao.BaseUserMapper;
import com.hcloud.microservice.org.facade.entity.BaseRole;
import com.hcloud.microservice.org.facade.entity.BaseRoleUser;
import com.hcloud.microservice.org.facade.entity.BaseUser;
import com.hcloud.microservice.org.facade.forms.BaseRoleAuthForm;
import com.hcloud.microservice.org.facade.forms.BaseRoleUserForm;
import com.hcloud.microservice.org.facade.forms.BaseUserAuthForm;
import com.hcloud.microservice.org.facade.forms.BaseUserForm;
import com.hcloud.microservice.org.service.BaseRoleService;
import com.hcloud.microservice.org.service.BaseUserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.net.util.Base64;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author xiezt
 */
@Slf4j
@Service("baseUserService")
public class BaseUserServiceImpl implements BaseUserService {


    /**
     * 角色未分配权限
     */
    private static final int SPECIAL_VAL=-2;

    @Resource
    private BaseUserMapper baseUserMapper;

    @Resource
    private BaseUserAuthorizationMapper baseUserAuthorizationMapper;

    @Resource
    private BaseRoleUserMapper baseRoleUserMapper;

    @Resource
    private BaseOrgAuthorizationMapper baseOrgAuthorizationMapper;

    @Autowired
    private BaseRoleService baseRoleService;

    @Override
    public int delByPrimaryKey(String guid) {
        return baseUserMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(BaseUser record) {
        String mingPWD = record.getPasswd();
        if(mingPWD!=null && !"".equals(mingPWD)){
//            String pwd = checkoutPwd(mingPWD);
//            record.setPasswd(Md5Util.md5(pwd));
            record.setPasswd(Md5Util.md5(mingPWD));
        }
        record.setGuid(GetUUID.getUUID());
        return baseUserMapper.insert(record);
    }

    @Override
    public int saveSelective(BaseUser record) {
        String mingPWD = record.getPasswd();
        if(mingPWD!=null && !"".equals(mingPWD)){
//            String pwd = checkoutPwd(mingPWD);
//            record.setPasswd(Md5Util.md5(pwd));
            record.setPasswd(Md5Util.md5(mingPWD));
        }
        record.setGuid(GetUUID.getUUID());
        return baseUserMapper.insertSelective(record);
    }

    @Override
    public BaseUser queryByPrimaryKey(String guid) {
        return baseUserMapper.selectByPrimaryKey(guid);
    }

    @Override
    public int modifyByPrimaryKeySelective(BaseUser record) {
        log.info("BaseUser====================>{}",record);
        String mingPWD = record.getPasswd();
        if(mingPWD!=null && !"".equals(mingPWD)){
//            String pwd = checkoutPwd(mingPWD);
//            record.setPasswd(Md5Util.md5(pwd));
            record.setPasswd(Md5Util.md5(mingPWD));
        }
        return baseUserMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(BaseUser record) {
        //TODO empty implements
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(BaseUser record) {
        return baseUserMapper.updateByPrimaryKey(record);
    }

    /**
     * 批量插入用户权限信息
     *
     * @param list
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int batchSaveUserAuthInfo(String userId,String orgGuid,List<BaseUserAuthForm> list) {
        List<BaseUserAuthForm> insertList = Lists.newArrayList();
        if(null!=list && list.size()>0) {
            baseUserAuthorizationMapper.delUserAuthInfoByUserId(userId, orgGuid);
            for (BaseUserAuthForm bean : list) {
                BaseUserAuthForm tmp = new BaseUserAuthForm();
                BeanCopyUtil.copyBean(bean, tmp);
                tmp.setFkAuthGuid(bean.getId());
                insertList.add(tmp);
            }
        }
        return baseUserAuthorizationMapper.batchSaveUserAuthInfo(insertList);
    }

    /**
     * 分页查询用户信息
     *
     * @param baseUserForm
     * @return
     */
    @Override
    public PageInfo searchUserInfoByPage(BaseUserForm baseUserForm) {
        PageBean pageBean = new PageBean(baseUserForm);
        PageHelper.startPage(baseUserForm.getPageNo(),baseUserForm.getPageSize());
        Page<BaseUserBO> page = baseUserMapper.searchUserInfoByPage(pageBean);
        return new PageInfo(page);
    }

    /**
     * 通过用户id查询用户权限信息
     *
     * @param userId
     * @param orgGuid
     * @return
     */
    @Override
    public List<BaseUserAuthForm> searchUserAuthInfo(String userId, String orgGuid) {
        List<BaseUserAuthForm> resultList = new ArrayList<>();
        //TODO 查询系统全部权限
        List<BaseOrgAuthBO> orgAuthList = baseOrgAuthorizationMapper.searchAllOrgAuthInfo(orgGuid);
        //TODO 查询用户权限信息
        List<String> list = baseUserAuthorizationMapper.searchUserAuthInfoByUserId(userId,orgGuid);
        for (BaseOrgAuthBO bo:orgAuthList) {
            String authStr = bo.getId();
            BaseUserAuthForm baseUserAuthForm = new BaseUserAuthForm();
            BeanUtils.copyProperties(bo,baseUserAuthForm);
            baseUserAuthForm.setFkUserGuid(userId);
            if(list.contains(authStr)){
                baseUserAuthForm.setIsChecked(1);
            }else{
                baseUserAuthForm.setIsChecked(0);
            }
            resultList.add(baseUserAuthForm);
        }
        return resultList;
    }

    /**
     * 通过用户id删除用户权限信息
     *
     * @param userId
     * @return
     */
    @Override
    public int delUserAuthInfoByUserId(String userId,String orgGuid) {
        return baseUserAuthorizationMapper.delUserAuthInfoByUserId(userId,orgGuid);
    }

    /**
     * 查询用户角色信息，通过用户查询角色，通过角色查用户
     *
     * @param baseRoleUserForm
     * @return
     */
    @Override
    public PageInfo searchRoleUserInfoById(BaseRoleUserForm baseRoleUserForm) {
        PageBean pageBean = new PageBean(baseRoleUserForm);
        PageHelper.startPage(baseRoleUserForm.getPageNo(),baseRoleUserForm.getPageSize());
        Page<BaseRoleUserBO> page = baseRoleUserMapper.searchRoleUserInfoByPage(pageBean);
        if(null!=page && page.size()>0){
            List<BaseRoleUserForm> pages = BeanCopyUtil.convertList(page,BaseRoleUserForm.class);
            return new PageInfo(pages);
        }
        return null;
    }

    /**
     * 分配用户角色信息
     *
     * @param baseUserForm
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void distributionUserAuthInfo(BaseUserForm baseUserForm) {
        String outRoleId = null;
        //添加企业信息管理员处理流程
        if(null!=baseUserForm.getLoginOrgGuid() && !"".equals(baseUserForm.getLoginOrgGuid())){
            BaseRole role =  baseRoleService.queryByPrimaryKey(baseUserForm.getRoleId());
            if(role!=null){
                role.setFkOrgGuid(baseUserForm.getFkOrgGuid());
                role.setGuid(GetUUID.getUUID());
                role.setCreateUser(baseUserForm.getCreateUser());
                role.setCreateTime(new Date());
                role.setModifiedUser(baseUserForm.getModifiedUser());
                role.setModifiedTime(new Date());
                int i = baseRoleService.saveSelective(role);
                if(i>0){
                    outRoleId = role.getGuid();
                }
            }
        }
        //添加普通用户角色关联信息
        BaseRoleUser baseRoleUser = new BaseRoleUser();
        baseRoleUser.setGuid(GetUUID.getUUID());
        //添加企业信息管理员处理流程
        if(null!=baseUserForm.getLoginOrgGuid() && !"".equals(baseUserForm.getLoginOrgGuid())){
            baseRoleUser.setFkRoleGuid(outRoleId);
        }else{
            baseRoleUser.setFkRoleGuid(baseUserForm.getRoleId());
        }
        baseRoleUser.setFkUserGuid(baseUserForm.getGuid());
        baseRoleUser.setFkOrgGuid(baseUserForm.getFkOrgGuid());
        baseRoleUser.setFkProductGuid(baseUserForm.getFkProductGuid());
        baseRoleUserMapper.insertSelective(baseRoleUser);
        //TODO 添加用户权限信息
        this.batchSaveUserAuthInfo(baseUserForm.getGuid(),baseUserForm.getFkOrgGuid(),baseUserForm.getAuthList());
    }

    /**
     * 查询用户信息
     *
     * @param account
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public BaseUserForm searchUserInfoByAccount(String account,String password) {
        BaseUserForm resultBean = new BaseUserForm();
        BaseUser baseUsers =  baseUserMapper.getUserInfoByAccount(account);
        if(null!=baseUsers){
            log.info("login password===================>{}",checkoutPwd(password));
            String loginPwd = checkoutPwd(password);
            String pwd = Md5Util.md5(loginPwd);
            Integer isForbid = baseUsers.getIsForbid();
            if(isForbid==1){
                resultBean.setResultEnum(ResultEnum.LOGIN_FORBIDED);
                return resultBean;
            }
            if(pwd.equalsIgnoreCase(baseUsers.getPasswd())){
                BeanCopyUtil.copyBean(baseUsers,resultBean);
                //TODO 更新登录时间
                BaseUser baseUser = new BaseUser();
                baseUser.setGuid(baseUsers.getGuid());
                baseUser.setFkOrgGuid(baseUsers.getFkOrgGuid());
                baseUser.setLastLoginTime(new Date());
                baseUserMapper.updateByPrimaryKeySelective(baseUser);
                log.info("======================>{}",resultBean);
                resultBean.setResultEnum(ResultEnum.SUCCESS);
                return resultBean;
            }
            resultBean.setResultEnum(ResultEnum.LOGIN_PWD_MISS);
            return resultBean;
        }
        resultBean.setResultEnum(ResultEnum.LOGIN_ACCOUNT_MISS);
        return resultBean;
    }

    /**
     * 查询用户菜单信息
     *
     * @param userId
     * @param orgGuid
     * @return
     */
    @Override
    public List<BaseUserAuthForm> getUserMenusInfo(String userId, String orgGuid) {
        List<BaseUserAuthForm> resultList = Lists.newArrayList();
        List<BaseUserAuthForm> tmpList = this.searchUserAuthInfo(userId,orgGuid);
        if(null!=tmpList && tmpList.size()>0){
            for (BaseUserAuthForm bean:tmpList) {
                if(bean.getNodeType()==0){
                    resultList.add(bean);
                }
            }
        }
        return resultList;
    }

    /**
     * 查询功能权限通过菜单权限
     *
     * @param userId
     * @param orgGuid
     * @param authId
     * @return
     */
    @Override
    public List<BaseUserAuthForm> getSubAuthById(String userId, String orgGuid, String authId) {
        List<BaseUserAuthForm> resultList = Lists.newArrayList();
        List<BaseUserAuthForm> tmpList = this.searchUserAuthInfo(userId,orgGuid);
        if(null!=tmpList && tmpList.size()>0){
            for (BaseUserAuthForm bean:tmpList) {
                if(authId.equals(bean.getParentGuid()) && bean.getIsChecked()==1){
                    resultList.add(bean);
                }
            }
        }
        return resultList;
    }

    /**
     * 查询重复用户信息
     *
     * @param account
     * @return
     */
    @Override
    public boolean searchRepeatUserInfo(String account) {
        BaseUserForm baseUserForm = new BaseUserForm();
        baseUserForm.setAccount(account);
        PageBean pageBean = new PageBean(baseUserForm);
        Page<BaseUserBO> page = baseUserMapper.searchUserInfoByPage(pageBean);
        if(null!=page && page.size()>0){
            return true;
        }
        return false;
    }

    /**
     * 新增用户信息（包含用户属于哪个角色，用户拥有的权限）
     *
     * @param baseUserForm
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int addUserInfoForAll(BaseUserForm baseUserForm) {
        if(null!=baseUserForm){
            //TODO 添加用户信息
            String orgGuid = baseUserForm.getFkOrgGuid();
            String roleId = baseUserForm.getRoleId();
            String loginOrgGuid = baseUserForm.getLoginOrgGuid();
            //初始化企业管理员用户信息
            if(null!=loginOrgGuid && !"".equals(loginOrgGuid)){
                orgGuid = loginOrgGuid;
            }
            List<BaseRoleAuthForm> roleAuthList = baseRoleService.searchOnlyRoleAuthInfo(roleId,orgGuid);
            if(roleAuthList==null || roleAuthList.size()==0){return SPECIAL_VAL;}
            BaseUser baseUser = new BaseUser();
            BeanUtils.copyProperties(baseUserForm,baseUser);
            int i = this.saveSelective(baseUser);
            //TODO 添加用户角色关系,并添加用户权限信息
            if(i>0){
                String guid = baseUser.getGuid();
                if(roleId!=null){
                    //TODO 添加用户权限
                    List<BaseUserAuthForm> userAuthList = BeanCopyUtil.convertList(roleAuthList,BaseUserAuthForm.class);
                    //添加企业管理员用户
                    if(null!=loginOrgGuid && !"".equals(loginOrgGuid)){
                        for (BaseUserAuthForm bean:userAuthList) {
                            bean.setFkOrgGuid(baseUserForm.getFkOrgGuid());
                        }
                    }
                    //TODO 添加用户Guid
                    for (BaseUserAuthForm bo: userAuthList) {
                        bo.setFkUserGuid(guid);
                    }
                    baseUserForm.setGuid(guid);
                    baseUserForm.setFkProductGuid(GlobleConstant.PRODUCT_GUID);
                    baseUserForm.setAuthList(userAuthList);
                    this.distributionUserAuthInfo(baseUserForm);
                }
               return 1;
            }
        }
        return 0;
    }

    /**
     * 修改用户信息（包含用户属于哪个角色，用户拥有的权限）
     *
     * @param baseUserForm
     * @return
     */
    @Override
    public int modifyUserInfoForAll(BaseUserForm baseUserForm) {
        if(null!=baseUserForm){
            //TODO 修改用户信息
            BaseUser baseUser = new BaseUser();
            BeanUtils.copyProperties(baseUserForm,baseUser);
            int i = this.modifyByPrimaryKeySelective(baseUser);
            if(i>0){
                //TODO 更新用户角色关系
                String userId = baseUserForm.getGuid();
                String roleId = baseUserForm.getRoleId();
                String orgGuid = baseUserForm.getFkOrgGuid();
                baseRoleUserMapper.delRoleUserInfoByUserId(userId,orgGuid);
                //TODO 添加用户权限
                List<BaseRoleAuthForm> roleAuthList = baseRoleService.searchOnlyRoleAuthInfo(roleId,orgGuid);
                List<BaseUserAuthForm> userAuthList = BeanCopyUtil.convertList(roleAuthList,BaseUserAuthForm.class);
                //TODO 添加用户Guid
                for (BaseUserAuthForm bo: userAuthList) {
                    bo.setFkUserGuid(userId);
                }
                baseUserForm.setFkProductGuid(GlobleConstant.PRODUCT_GUID);
                baseUserForm.setAuthList(userAuthList);
                this.distributionUserAuthInfo(baseUserForm);
                return 1;
            }
        }
        return 0;
    }

    /**
     * 查看用户账户是否有重复的
     * @param baseUserForm
     * @return
     */
    @Override
    public boolean isAccountsValid(BaseUserForm baseUserForm) {
        if(null!=baseUserForm && StringUtils.isNotEmpty(baseUserForm.getAccount())){
            BaseUser baseUser = this.baseUserMapper.selectAccountNotSelf(baseUserForm);
            return baseUser != null ? false : true;
        }
        return true;
    }

    /**
     * 登录解密方式
     * @param pwd
     * @return
     */
    private String checkoutPwd(String pwd){
        //需要进行对应的接密操作
        //1.获取盐值
        String salt = pwd.substring(0, 5);
        String key = "b8d11ee289394be688ef3a4f6968efed";
        String time1 = Md5Util.md5((key).substring(0, 8)).substring(0, 5);
        //做超时判断
        if (!time1.equalsIgnoreCase(salt)) {
            throw new RuntimeException();
        }
        //2.去除盐值
        String replace = pwd.replace(salt, "");
        //3.还原base64前的密码值
        try {
            return new String(Base64.decodeBase64(replace), "utf-8");
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException();
        }
    }
}
