package com.hcloud.microservice.org.service.impl;

import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microservice.org.TestInit;
import com.hcloud.microservice.org.bo.BaseAuthBO;
import com.hcloud.microservice.org.facade.entity.BaseAuthorization;
import com.hcloud.microservice.org.facade.forms.BaseAuthForm;
import com.hcloud.microservice.org.service.BaseAuthorizationService;
import lombok.extern.slf4j.Slf4j;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

@Slf4j
@RunWith(SpringRunner.class)
@SpringBootTest
@Transactional
public class BaseAuthorizationServiceImplTest extends TestInit {

    @Autowired
    private BaseAuthorizationService baseAuthorizationService;

    private BaseAuthorization auth;

    @Before
    public void setUp() throws Exception {
        auth = new BaseAuthorization();
        auth.setParentGuid("1f95b74e91f146ebaee5ad0a46742aad");
        auth.setFkProductGuid(super.FK_PRODUCT_GUID);
        auth.setFunName("禁用平台用户信息");
        auth.setAuthStr("platform:sys:deluser");
        auth.setRemark("禁用平台用户信息");
        auth.setOrderby(96);
        auth.setNodeType(1);
        auth.setIsForbid(0);
        auth.setState(1);
        auth.setCreateUser(super.FK_CREATE_USER);
        auth.setCreateTime(new Date());
        auth.setModifiedUser(super.FK_MODIFIED_USER);
        auth.setModifiedTime(new Date());
    }

    @After
    public void tearDown() throws Exception {
    }

    @Test
    public void saveSelective() {
      int i=  baseAuthorizationService.saveSelective(auth);
        Assert.assertNotEquals(0,i);
    }

    @Test
    public void queryByPrimaryKey() {
        BaseAuthorization bean =
                baseAuthorizationService.queryByPrimaryKey("903471b6473d464098b8d85c8db8885d");
        Assert.assertNotNull(bean);
    }

    @Test
    public void modifyByPrimaryKeySelective() {
        BaseAuthorization bean = new BaseAuthorization();
        bean.setGuid("903471b6473d464098b8d85c8db8885d");
        bean.setIsForbid(1);
        int i = baseAuthorizationService.modifyByPrimaryKeySelective(auth);
        Assert.assertNotEquals(0,i);
        BaseAuthorization beans =
                baseAuthorizationService.queryByPrimaryKey("903471b6473d464098b8d85c8db8885d");
        Assert.assertEquals(new Integer(1),beans.getIsForbid());
    }

    @Test
    public void searchBaseAuthInfoByPage() {
        BaseAuthForm baseAuthForm = new BaseAuthForm();
        //baseAuthForm.setFunName("权限管理");
        baseAuthForm.setFileUrl("sys/auth/managerPage");
        PageInfo pageInfo = baseAuthorizationService.searchBaseAuthInfoByPage(baseAuthForm);
        List page = pageInfo.getList();
        BaseAuthBO pps = (BaseAuthBO) page.get(0);
        log.info("funName=============>{}",pps.getFunName());
        log.info("fileUrl=============>{}",pps.getFileUrl());
        Assert.assertEquals(1,pageInfo.getTotal());
        Assert.assertEquals(pps.getFileUrl(),"sys/auth/managerPage");
    }

    @Test
    public void searchMenuAuthInfo() {
        List list = baseAuthorizationService.searchMenuAuthInfo();
        for (Object o:list) {
            BaseAuthBO bo = (BaseAuthBO) o;
            log.info("bo info==============>{}",bo);
        }
        Assert.assertEquals(list.size(),3);
    }
}