package com.hcloud.microservice.org.service.impl;

import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microserver.commoncore.util.Md5Util;
import com.hcloud.microservice.org.TestInit;
import com.hcloud.microservice.org.facade.entity.BaseUser;
import com.hcloud.microservice.org.facade.forms.BaseRoleUserForm;
import com.hcloud.microservice.org.facade.forms.BaseUserAuthForm;
import com.hcloud.microservice.org.facade.forms.BaseUserForm;
import com.hcloud.microservice.org.service.BaseUserService;
import lombok.extern.slf4j.Slf4j;
import org.bouncycastle.jcajce.provider.digest.MD5;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static org.junit.Assert.*;

@Slf4j
@RunWith(SpringRunner.class)
@SpringBootTest
//@Transactional
public class BaseUserServiceImplTest extends TestInit {

    @Autowired
    private BaseUserService baseUserService;

    @Test
    public void saveSelective() {
        BaseUser baseUser = new BaseUser();
        baseUser.setGuid(GetUUID.getUUID());
        baseUser.setFkOrgGuid(super.FK_ORG_GUID);
        baseUser.setFkProductGuid(super.FK_PRODUCT_GUID);
        baseUser.setAccount("999999");
        baseUser.setUserName("测试类生成用户2");
        baseUser.setNickname("安全测试通过2");
        baseUser.setDicSexCode(1);
        baseUser.setPasswd(Md5Util.toMD5("123456"));
        baseUser.setPid("5201121478956421");
        baseUser.setPhoto("images/sss.png");
        baseUser.setMobileNo("13809548362");
        baseUser.setIsModifiedPwd(0);
        baseUser.setLastChangePwdTime(new Date());
        baseUser.setInputPwdErrorNum(0);
        baseUser.setLastLoginTime(new Date());
        baseUser.setLockTime(0);
        baseUser.setIsForbid(0);
        baseUser.setRemark("测试类生成用户信息");
        baseUser.setState(1);
        baseUser.setCreateUser(super.FK_CREATE_USER);
        baseUser.setCreateTime(new Date());
        baseUser.setModifiedUser(super.FK_MODIFIED_USER);
        baseUser.setModifiedTime(new Date());
        int i = baseUserService.saveSelective(baseUser);
        Assert.assertEquals(1,i);
    }

    @Test
    public void queryByPrimaryKey() {
        String guid = "ead1f9f3c69544a5a523966c8b723b18";
        BaseUser baseUser = baseUserService.queryByPrimaryKey(guid);
        Assert.assertNotNull(baseUser);
        Assert.assertEquals(guid,baseUser.getGuid());
    }

    @Test
    public void batchSaveUserAuthInfo() {
        String guid = "ead1f9f3c69544a5a523966c8b723b18";
        List<BaseUserAuthForm> list = new ArrayList<>();
        for (int i = 0; i <20 ; i++) {
            BaseUserAuthForm baseUserAuthForm = new BaseUserAuthForm();
            baseUserAuthForm.setFkUserGuid(guid);
            baseUserAuthForm.setFkAuthGuid(GetUUID.getUUID());
            baseUserAuthForm.setFkProductGuid(super.FK_PRODUCT_GUID);
            baseUserAuthForm.setFkOrgGuid(super.FK_ORG_GUID);
            list.add(baseUserAuthForm);
        }
        int i = baseUserService.batchSaveUserAuthInfo(guid,super.FK_ORG_GUID,list);
        Assert.assertEquals(20,i);
    }

    @Test
    public void searchUserInfoByPage() {
        BaseUserForm baseUserForm = new BaseUserForm();
        baseUserForm.setFkOrgGuid(super.FK_ORG_GUID);
        baseUserForm.setUserName("测试");
        baseUserForm.setIsForbid(0);
        baseUserForm.setBeginDate("2019-01-05");
        baseUserForm.setEndDate("2019-01-05");
        PageInfo pageInfo = baseUserService.searchUserInfoByPage(baseUserForm);
        log.info("object=====================>{}",pageInfo.getList().get(0));
        Assert.assertNotNull(pageInfo.getList());
        Assert.assertEquals(2,pageInfo.getTotal());
    }

    @Test
    public void searchUserAuthInfo() {
        String guid = "ead1f9f3c69544a5a523966c8b723b18";
       List<BaseUserAuthForm> list =  baseUserService.searchUserAuthInfo(guid,super.FK_ORG_GUID);
       Assert.assertNotNull(list);
       Assert.assertEquals(20,list.size());
    }

    @Test
    public void delUserAuthInfoByUserId() {
        String guid = "ead1f9f3c69544a5a523966c8b723b18";
        int i =  baseUserService.delUserAuthInfoByUserId(guid,super.FK_ORG_GUID);
        Assert.assertEquals(20,i);
    }

    @Test
    public void searchRoleUserInfoById() {
        BaseRoleUserForm baseRoleUserForm = new BaseRoleUserForm();
        baseRoleUserForm.setFkOrgGuid(super.FK_ORG_GUID);
//        baseRoleUserForm.setFkUserGuid("ead1f9f3c69544a5a523966c8b723b18");
        baseRoleUserForm.setFkRoleGuid("8e1ccb4116454ebbbb5f0970422253b0");
        PageInfo pageInfo = baseUserService.searchRoleUserInfoById(baseRoleUserForm);
        log.info("======================>{}",pageInfo.getList().get(0));
        Assert.assertNotNull(pageInfo.getList());
        Assert.assertEquals(1,pageInfo.getTotal());
    }

    @Test
    public void distributionUserAuthInfo(){
        String guid = "ead1f9f3c69544a5a523966c8b723b18";
        List<BaseUserAuthForm> list = new ArrayList<>();
        for (int i = 0; i <20 ; i++) {
            BaseUserAuthForm baseUserAuthForm = new BaseUserAuthForm();
            baseUserAuthForm.setFkUserGuid(guid);
            baseUserAuthForm.setFkAuthGuid(GetUUID.getUUID());
            baseUserAuthForm.setFkProductGuid(super.FK_PRODUCT_GUID);
            baseUserAuthForm.setFkOrgGuid(super.FK_ORG_GUID);
            list.add(baseUserAuthForm);
        }
        BaseUserForm baseUserForm = new BaseUserForm();
        baseUserForm.setGuid(guid);
        baseUserForm.setFkOrgGuid(super.FK_ORG_GUID);
        baseUserForm.setFkProductGuid(super.FK_PRODUCT_GUID);
        baseUserForm.setRoleId("8e1ccb4116454ebbbb5f0970422253b0");
        baseUserForm.setAuthList(list);
        baseUserService.distributionUserAuthInfo(baseUserForm);
    }
}